using System;
using System.Collections.Generic;

using Server;
using Server.Items;
using Server.Mobiles;

namespace Server.Items
{
	public struct SetPart
	{
		public bool Valid { get { return (m_TypeOf != null); } }

		private Type m_TypeOf;
		public Type TypeOf { get { return m_TypeOf; } }

		private string m_Name;
		public string Name { get { return m_Name; } }

		public SetPart(Type typeOf)
			: this(typeOf, typeOf.Name)
		{ }

		public SetPart(Type typeOf, string name)
		{
			m_TypeOf = typeOf;

			if (String.IsNullOrEmpty(name))
				name = typeOf.Name;

			m_Name = name;
		}

		public override int GetHashCode()
		{
			return m_TypeOf.GetHashCode();
		}

		public override bool Equals(object obj)
		{
			return base.Equals(obj);
		}

		public static bool operator ==(SetPart x, SetPart y)
		{
			return (x.TypeOf == y.TypeOf);
		}

		public static bool operator !=(SetPart x, SetPart y)
		{
			return (x.TypeOf != y.TypeOf);
		}
	}

	public abstract class SetBonus
	{
		public abstract int CountReq { get; }

		public SetBonus()
		{ }

		public abstract void Apply(Mobile owner);
		public abstract void Remove(Mobile owner);
	}

	public abstract class ItemSet
	{
		public abstract SetPart[] Parts { get; }
		public abstract SetBonus[] Bonuses { get; }

		public abstract int Hue { get; }

		public abstract string Name { get; }

		public Type TypeOf { get { return GetType(); } }

		public bool IsActive(Mobile owner)
		{
			return (Invalidate(owner, false) > 0);
		}

		public int Invalidate(Mobile owner, bool refreshProps, out Dictionary<SetPart, Item> equippedParts)
		{
			return Invalidate(owner.Items, refreshProps, out equippedParts);
		}

		public int Invalidate(List<Item> items, bool refreshProps, out Dictionary<SetPart, Item> equippedParts)
		{
			equippedParts = new Dictionary<SetPart, Item>();

			if (items == null)
				return 0;

			//Console.WriteLine("Set Invalidation...");

			foreach (Item item in items)
			{
				if (item == null || item.Deleted)
					continue;

				for (int i = 0; i < Parts.Length; i++)
				{
					Type type = item.GetType();

					if (type == Parts[i].TypeOf)
					{
						//Console.WriteLine("Set Part Is Valid: {0} - {1}", item.Name, type.Name);

						if (IsEquipped(item))
						{
							//Console.WriteLine("Set Part Is Equipped: {0} - {1}", item.Name, type.Name);

							if (!equippedParts.ContainsKey(Parts[i]))
							{
								equippedParts.Add(Parts[i], item);
								break;
							}
						}
						else
						{
							//Console.WriteLine("Set Part Is Not Equipped: {0} - {1}", item.Name, type.Name);

							if (equippedParts.ContainsKey(Parts[i]))
							{
								equippedParts.Remove(Parts[i]);
								break;
							}
						}
					}
				}
			}

			OnInvalidated(equippedParts);

			if (refreshProps)
				InvalidateItemProperties(new List<Item>(equippedParts.Values));

			return equippedParts.Count;
		}

		public int Invalidate(Mobile owner, bool refreshProps)
		{
			return Invalidate(owner.Items, refreshProps);
		}

		public int Invalidate(List<Item> items, bool refreshProps)
		{
			if (items == null)
				return 0;

			//Console.WriteLine("Set Invalidation...");

			List<Item> equippedParts = new List<Item>();

			foreach (Item item in items)
			{
				if (item == null || item.Deleted)
					continue;

				for (int i = 0; i < Parts.Length; i++)
				{
					Type type = item.GetType();

					if (type == Parts[i].TypeOf)
					{
						//Console.WriteLine("Set Part Is Valid: {0} - {1}", item.Name, type.Name);

						if (IsEquipped(item))
						{
							//Console.WriteLine("Set Part Is Equipped: {0} - {1}", item.Name, type.Name);

							if (!equippedParts.Contains(item))
							{
								equippedParts.Add(item);
								break;
							}
						}
						else
						{
							//Console.WriteLine("Set Part Is Not Equipped: {0} - {1}", item.Name, type.Name);

							if (equippedParts.Contains(item))
							{
								equippedParts.Remove(item);
								break;
							}
						}
					}
				}
			}

			OnInvalidated(equippedParts);

			if (refreshProps)
				InvalidateItemProperties(equippedParts);

			return equippedParts.Count;
		}

		public static bool IsEquipped(Item item)
		{
			if (item.Parent != null)
			{
				object parent = item.Parent;

				if (parent is Mobile)
				{
					Mobile parent_m = (Mobile)parent;

					if (parent_m.Items.Contains(item))
						return true;
				}
			}

			return false;
		}

		public void InvalidateItemProperties(List<Item> items)
		{
			foreach (Item item in items)
				item.InvalidateProperties();
		}

		public virtual void OnInvalidated(Dictionary<SetPart, Item> equippedParts) { }

		public virtual void OnInvalidated(List<Item> equippedParts) { }

		public void Activate(Mobile owner, int partCount)
		{
			//Console.WriteLine("Set Activate: {0} - {1} (Part Count: {2})", Name, TypeOf.Name, partCount);

			foreach (SetBonus bonus in Bonuses)
			{
				if (partCount >= bonus.CountReq)
				{
					//Console.WriteLine("Set Bonus Apply: {0} - Rank {1}", bonus.TypeOf.Name, bonus.Rank);

					bonus.Apply(owner);
				}
			}

			OnActivate(owner, partCount);
		}

		public virtual void OnActivate(Mobile owner, int partCount) { }

		public void Deactivate(Mobile owner, int partCount)
		{
			//Console.WriteLine("Set Deactivate: {0} - {1} (Part Count: {2})", Name, TypeOf.Name, partCount);

			foreach (SetBonus bonus in Bonuses)
			{
				if (partCount < bonus.CountReq)
				{
					//Console.WriteLine("Set Bonus Remove: {0} - Rank {1}", bonus.TypeOf.Name, bonus.Rank);

					bonus.Remove(owner);
				}
			}

			OnDeactivate(owner, partCount);
		}

		public virtual void OnDeactivate(Mobile owner, int partCount) { }

		public void PartAdded(Mobile owner, Item part)
		{
			List<Item> merge = new List<Item>(owner.Items);

			if (!merge.Contains(part))
				merge.Add(part);

			int partCount = Invalidate(merge, true);

			Activate(owner, partCount);

			OnPartAdded(owner, part);

			owner.InvalidateProperties();
		}

		public virtual void OnPartAdded(Mobile owner, Item part) { }

		public void PartRemoved(Mobile owner, Item part)
		{
			List<Item> merge = new List<Item>(owner.Items);

			if (merge.Contains(part))
				merge.Remove(part);

			int partCount = Invalidate(merge, true);

			Deactivate(owner, partCount);

			OnPartRemoved(owner, part);

			owner.InvalidateProperties();
		}

		public virtual void OnPartRemoved(Mobile owner, Item part) { }
	}
}